/**
 * @author Roland Toth (tpr)
 * v0.1
 * 12/08/2012
 * http://rolandtoth.hu/TOCsic
 * http://rolandtoth.posterous.com/introducing-tocsic-a-powerful-jquery-toc-plug
 */

(function($){
    "use strict";
    $.fn.extend({
        //plugin name - TOCsic
        TOCsic: function(opt) {
 
            //Settings list and the default values
            var defaults = {
                containerClass: 'TOCsic',
                listClass: 'TOCsic',
                attachTo: 'body',
                scrollSpeed: 500,
                scrollPageTop: false,
                title: '',
                titleTag: 'h2',
                topLinkText: '',
                topLinkTitle: 'Go to top',
                topLinkBeforeTarget: true,
                oneTopLinkOnly: false,
                topLinkVisibleAfter: 300,
                topLinkFadeTime: 200,
                afterChar: '',
                beforeChar: '',
                addLevelClass: true
            };
             
            var options = $.extend(defaults, opt);
            
            if($(options.attachTo).length === 0) { return false; }
            
            if(isNaN(options.topLinkVisibleAfter)) { options.topLinkVisibleAfter = 200; }
            if(isNaN(options.topLinkFadeTime)) { options.topLinkFadeTime = 100; }
            if(isNaN(options.scrollSpeed)) { options.scrollSpeed = 500; }
            if(options.afterChar !== '') { options.afterChar = '<span class="afterChar">' + options.afterChar + '</span>'; }
            if(options.beforeChar !== '') { options.beforeChar = '<span class="beforeChar">' + options.beforeChar + '</span>'; }
            
            // create topLink target
            var topLinkTarget = '<a href="#" class="' + options.containerClass + '"  id="' + options.containerClass + '"></a>';
            
            // add container div and ul (if scroll to top is the target and if the title is set)
            if(options.title !== "") {
                options.title = "<" + options.titleTag + " class='TOCtitle'>" + options.title + topLinkTarget + "</" + options.titleTag + ">";
                $(options.attachTo).prepend(options.title + "<div class=" + options.containerClass + " ><ul class=" + options.listClass + " ></ul></div>");
            } else {
                $(options.attachTo).prepend(options.title + "<div class=" + options.containerClass + " >" + topLinkTarget + "<ul class=" + options.listClass + " ></ul></div>");
            }
            
            var counter = 0;
            this.each(function() {
                var o = options;
                var target = $(this);
                var linkName = "toc-" + target.text().replace(/[^a-zA-Z]+/g, '') + counter++;
                
                // add TOC elements to the container
                var levelClasses = "";
                var currentClass = this.className;
                if(o.addLevelClass === true ) {
                    currentClass = (currentClass !== "" ? " toc-" + currentClass : "");
                    levelClasses = 'class="toc-' + this.tagName.toLowerCase() + currentClass + '"';
                }
                
                $('div.' + o.containerClass + ' ul').append("<li " + levelClasses + " ><a href='#" + linkName + "' title='' " + levelClasses + ">" + o.beforeChar + target.text() + o.afterChar + "</a></li>");
                
                // add placeholders to targets to scroll/jump to
                if(target.get(0).tagName === "A") {
                    // target is already a link, adding "id" attribute only
                    $(target).attr("id", linkName);
                } else {
                    // target is not a link, adding dummy link
                    $(target).append("<a href='#' id='" + linkName + "'></a>");
                }
                
                // add "go to top" links
                if(o.topLinkText !== '') {
                    var topLink = '<a href="#' + o.containerClass + '" title="' + o.topLinkTitle + '" class="topLink" >' + o.topLinkText + '</a>';
                    // add only one "go to top" link if oneTopLinkOnly is true
                    if(o.oneTopLinkOnly === true) {
                        if($('a.topLink').length === 0) {
                            if(o.topLinkBeforeTarget) {
                                $(o.attachTo).prepend(topLink);
                            } else {
                                $(o.attachTo).append(topLink);
                            }
                            
                            $('a.topLink').addClass('fixed').css('display', 'none');
                        }
                    // add several "go to top" links
                    } else {
                        if(o.topLinkBeforeTarget) {
                            $(target).children().first().before(topLink);
                        } else {
                            $(target).children().first().after(topLink);
                        }
                    }
                }
                
            });
            
            // add animation (disable scrolling if scrollSpeed is 0)
            if(options.scrollSpeed !== 0) {
                var op = jQuery('html, body');
                
                $("ul." + options.listClass + " a, a.topLink").click(function(e) {
                    e.preventDefault();
                    
                    // scroll to top if scrollPageTop is true
                    if(options.scrollPageTop === true && (e.target.className === "topLink" || e.target.className === "topLink fixed")) {
                        op.animate({scrollTop:0}, options.scrollSpeed);
                        return false;
                    }
                    
                    var full_url = this.href;
                    var parts = full_url.split("#");
                    var trgt = parts[1];
                    var target_top = $("a#" + trgt).offset().top - 5;
                    op.animate({scrollTop:target_top}, options.scrollSpeed);
                });
            }
            
            // make fixed topLink visible only if page is scrolled
            var checkPos = function () {
                if(options.oneTopLinkOnly === true) {
                    var topLink = $('a.topLink');
                    
                    var pageY = !isNaN(window.pageYOffset) ? window.pageYOffset : document.documentElement.scrollTop;
                    if (pageY > options.topLinkVisibleAfter) {
                        topLink.fadeIn(options.topLinkFadeTime);
                    } else {
                        topLink.fadeOut(options.topLinkFadeTime);
                    }
                }
            };
            
            if($('a.topLink').length === 1) {
                window.onscroll = checkPos;
            }
            
            checkPos();
        }
    });
})(jQuery);